@@ -2,7 +2,7 @@
 
 package Git::Hooks::CheckCommit;
 # ABSTRACT: Git::Hooks plugin to enforce commit policies
-$Git::Hooks::CheckCommit::VERSION = '1.10.0';
+$Git::Hooks::CheckCommit::VERSION = '1.11.0';
 use 5.010;
 use utf8;
 use strict;
@@ -10,6 +10,7 @@ use warnings;
 use Error ':try';
 use Git::Hooks qw/:DEFAULT :utils/;
 use Git::More::Message;
+use Data::Util qw(:check);
 use List::MoreUtils qw/any none/;
 
 my $PKG = __PACKAGE__;
@@ -204,15 +205,67 @@ sub signature_errors {
 sub commit_errors {
     my ($git, $commit) = @_;
 
+    return
+        match_errors($git, $commit) +
+        email_valid_errors($git, $commit) +
+        canonical_errors($git, $commit) +
+        signature_errors($git, $commit);
+}
+
+sub ref_errors {
+    my ($git, $ref) = @_;
+
     my $errors = 0;
 
-    $errors += match_errors($git, $commit);
+    state $codes;
 
-    $errors += email_valid_errors($git, $commit);
+    unless (ref $codes) {
+        $codes = [];
+      CODE:
+        foreach my $check ($git->get_config($CFG => 'check-ref')) {
+            my $code;
+            if ($check =~ s/^file://) {
+                $code = do $check;
+                unless ($code) {
+                    if (length $@) {
+                        $git->error($PKG, "couldn't parse check-ref file ($check): ", $@);
+                    } elsif (! defined $code) {
+                        $git->error($PKG, "couldn't read check-ref file ($check): ", $!);
+                    } else {
+                        $git->error($PKG, "check-ref file ($check) returned FALSE");
+                    }
+                    ++$errors;
+                    next CODE;
+                }
+            } else {
+                $code = eval $check; ## no critic (BuiltinFunctions::ProhibitStringyEval)
+                if (length $@) {
+                    $git->error($PKG, "couldn't parse check-ref value: ", $@);
+                    ++$errors;
+                    next CODE;
+                }
+            }
+            if (is_code_ref($code)) {
+                push @$codes, $code;
+            } else {
+                $git->error($PKG, "option check-ref must end with a code ref");
+                ++$errors;
+            }
+        }
+    }
 
-    $errors += canonical_errors($git, $commit);
-
-    $errors += signature_errors($git, $commit);
+    foreach my $code (@$codes) {
+        my $ok = eval { $code->($git, $ref) };
+        if (defined $ok) {
+            unless ($ok) {
+                $git->error($PKG, "error while evaluating check-ref");
+                ++$errors;
+            }
+        } elsif (length $@) {
+            $git->error($PKG, 'error while evaluating check-ref', $@);
+            ++$errors;
+        }
+    }
 
     return $errors;
 }
@@ -223,11 +276,12 @@ sub check_ref {
     my $errors = 0;
 
     foreach my $commit ($git->get_affected_ref_commits($ref)) {
-        commit_errors($git, $commit) == 0
-            or ++$errors;
+        $errors += commit_errors($git, $commit);
     }
 
-    return $errors == 0;
+    $errors += ref_errors($git, $ref);
+
+    return $errors;
 }
 
 sub check_pre_commit {
@@ -286,8 +340,7 @@ sub check_affected_refs {
     my $errors = 0;
 
     foreach my $ref ($git->get_affected_refs()) {
-        check_ref($git, $ref)
-            or ++$errors;
+        $errors += check_ref($git, $ref);
     }
 
     return $errors == 0;
@@ -329,7 +382,7 @@ Git::Hooks::CheckCommit - Git::Hooks plugin to enforce commit policies
 
 =head1 VERSION
 
-version 1.10.0
+version 1.11.0
 
 =head1 DESCRIPTION
 
@@ -383,7 +436,7 @@ option:
 
     git config --add githooks.plugin CheckCommit
 
-=for Pod::Coverage match_errors email_valid_errors canonical_errors identity_errors signature_errors spelling_errors pattern_errors subject_errors body_errors footer_errors commit_errors check_pre_commit check_post_commit check_ref check_affected_refs check_patchset
+=for Pod::Coverage match_errors email_valid_errors canonical_errors identity_errors signature_errors spelling_errors pattern_errors subject_errors body_errors footer_errors commit_errors ref_errors check_pre_commit check_post_commit check_ref check_affected_refs check_patchset
 
 =head1 NAME
 
@@ -505,6 +558,38 @@ signatures.
 
 This check is performed by the C<post-commit> local hook.
 
+=head2 githooks.checkcommit.check-ref CODESPEC
+
+If the above checks aren't enough you can use this option to define a custom
+code to check your commits during a B<git push>. The code may be specified
+directly as the option's value or you may specify it indirectly via the
+filename of a script. If the option's value starts with "file:", the
+remaining is treated as the script filename, which is executed by a B<do>
+command. Otherwise, the option's value is executed directly by an
+eval. Either way, the code must end with the definition of a routine, which
+will be called once for each reference being affected by the push with the
+following arguments:
+
+=over
+
+=item * B<GIT>
+
+The Git repository object used to grok information about the commit.
+
+=item * B<REF>
+
+The name of the reference being changed.
+
+=back
+
+The subroutine should return a boolean value indicating success. Any
+errors should be produced by invoking the B<Git::More::error> method.
+
+If the subroutine returns undef it's considered to have succeeded.
+
+If it raises an exception (e.g., by invoking B<die>) it's considered
+to have failed and a proper message is produced to the user.
+
 =head1 REFERENCES
 
 =over
